from typing import Any

from fastapi import APIRouter, Depends, Request

import schemas
from api import deps
from db.ckdb import CKDrive, get_ck_db
from db.redisdb import RedisDrive, get_redis_pool
from models.behavior_analysis import BehaviorAnalysis

router = APIRouter()


@router.post("/test")
async def test(
        request: Request,
        game: str,
        ckdb: CKDrive = Depends(get_ck_db),
        rdb: RedisDrive = Depends(get_redis_pool),
        current_user: schemas.UserDB = Depends(deps.get_current_user),
        analysis: BehaviorAnalysis = Depends(BehaviorAnalysis)) -> schemas.Msg:
    await analysis.init()
    query = analysis.scatter_model_sql()
    data = {
        'game': game,
        'analysis': analysis.game,
        'query': query
    }
    return schemas.Msg(code=0, msg='ok', data=data)
